import 'locale_setting.dart';
import 'resource.dart';
import 'text_size_scheme.dart';
import 'theme_color_scheme.dart';

/// 主题资源清单
class FThemeManifestInfo {
  /// 主题名称
  final String name;

  /// 描述
  final String description;

  /// 作者
  final String author;

  /// 版本号
  final String version;

  /// 版本类型
  final String versionType;

  /// 客户端适配最小版本号限制
  final String clientVersionMinimum;

  /// 适配平台集合
  final List<String> supportedPlatforms;

  /// 资源信息集合
  final List<FResourceInfo> resources;

  /// 语言配置
  final FThemeLocaleSettings localeSettings;

  /// 配色信息
  final FThemeColorScheme colorScheme;

  /// 配色(深色)信息
  final FThemeColorScheme? darkColorScheme;

  /// 文字尺寸组合
  final FThemeTextSizeScheme textSizeScheme;

  const FThemeManifestInfo({
    required this.name,
    this.description = "",
    this.author = "",
    required this.version,
    this.versionType = "flyinsono",
    this.clientVersionMinimum = "1.0.0",
    this.supportedPlatforms = const [],
    this.resources = const [],
    required this.localeSettings,
    required this.colorScheme,
    this.darkColorScheme,
    required this.textSizeScheme,
  });

  factory FThemeManifestInfo.fromJson(Map<String, dynamic> json) {
    List<dynamic> resourceJsonList = json['resources'];
    var resources =
        resourceJsonList.map((json) => FResourceInfo.fromJson(json)).toList();
    List<dynamic> supportedPlatformJsonList = json['supportedPlatforms'];
    List<String> supportedPlatformList =
        supportedPlatformJsonList.cast<String>();

    return FThemeManifestInfo(
      name: json['name'],
      description: json['description'],
      author: json['author'],
      version: json['version'],
      versionType: json['versionType'],
      clientVersionMinimum: json['clientVersionMinimum'],
      supportedPlatforms: supportedPlatformList,
      resources: resources,
      localeSettings: FThemeLocaleSettings.fromJson(json['localeSettings']),
      colorScheme: FThemeColorScheme.fromJson(json['colorScheme']),
      darkColorScheme: json['darkColorScheme'] != null
          ? FThemeColorScheme.fromJson(json['darkColorScheme'])
          : null,
      textSizeScheme: FThemeTextSizeScheme.fromJson(json['textSizeScheme']),
    );
  }
}
