import 'dart:typed_data';
import 'dart:ui' as ui show Codec;

import 'package:flutter/foundation.dart';
import 'package:flutter/rendering.dart';

import 'resource.dart';
import 'theme.dart';

/// 资源图片Provider
class FResourceImage extends ImageProvider<FResourceImage> {
  FResourceImage(this.key, {this.scale = 1.0});
  final FResourceKey key;
  final double scale;
  String _themeName = FTheme.ins.data.name;

  @override
  Future<FResourceImage> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<FResourceImage>(this);
  }

  @override
  ImageStreamCompleter load(FResourceImage key, decode) {
    return MultiFrameImageStreamCompleter(
      codec: _loadAsync(key, decode),
      scale: key.scale,
      debugLabel: 'FResourceImage(${describeIdentity(key.key)})',
    );
  }

  Future<ui.Codec> _loadAsync(
      FResourceImage key, DecoderCallback decode) async {
    assert(key == this);
    Uint8List bytes = await fRootBundle.loadAsBytes(this.key);
    return decode(bytes);
  }

  @override
  bool operator ==(Object other) {
    if (other.runtimeType != runtimeType) return false;
    return other is FResourceImage &&
        other.key == key &&
        other.scale == scale &&
        other._themeName == _themeName;
  }

  @override
  int get hashCode => hashValues(key.hashCode, scale);

  @override
  String toString() =>
      '${objectRuntimeType(this, 'FResourceImage')}(${describeIdentity(key)}, scale: $scale)';
}
