/// 主题文字尺寸组合
class FThemeTextSizeScheme {
  /// 大标题文字尺寸
  final double headlineLargeSize;

  /// 标题文字尺寸
  final double headlineSize;

  /// 小标题文字尺寸
  final double headlineSmallSize;

  /// 正文文字尺寸
  final double bodySize;

  /// 按钮文字尺寸
  final double buttonSize;

  /// 提示文字尺寸
  final double tipsSize;

  /// 提示（小）文字尺寸
  final double tipsSmallSize;

  factory FThemeTextSizeScheme({
    double? headlineLargeSize,
    double? headlineSize,
    double? headlineSmallSize,
    double? bodySize,
    double? buttonSize,
    double? tipsSize,
    double? tipsSmallSize,
  }) {
    return FThemeTextSizeScheme.raw(
      headlineLargeSize: headlineLargeSize ?? _FallbackValues.HeadlineLargeSize,
      headlineSize: headlineSize ?? _FallbackValues.HeadlineSize,
      headlineSmallSize: headlineSmallSize ?? _FallbackValues.HeadlineSmallSize,
      bodySize: bodySize ?? _FallbackValues.BodySize,
      buttonSize: buttonSize ?? _FallbackValues.ButtonSize,
      tipsSize: tipsSize ?? _FallbackValues.TipsSize,
      tipsSmallSize: tipsSmallSize ?? _FallbackValues.TipsSmallSize,
    );
  }

  FThemeTextSizeScheme.raw({
    required this.headlineLargeSize,
    required this.headlineSize,
    required this.headlineSmallSize,
    required this.bodySize,
    required this.buttonSize,
    required this.tipsSize,
    required this.tipsSmallSize,
  });

  factory FThemeTextSizeScheme.fromJson(Map<String, dynamic>? map) {
    if (map == null) return FThemeTextSizeScheme();

    return FThemeTextSizeScheme(
      headlineLargeSize: map['headlineLargeSize'] as double?,
      headlineSize: map['headlineSize'] as double?,
      headlineSmallSize: map['headlineSmallSize'] as double?,
      bodySize: map['bodySize'] as double?,
      buttonSize: map['buttonSize'] as double?,
      tipsSize: map['tipsSize'] as double?,
      tipsSmallSize: map['tipsSmallSize'] as double?,
    );
  }
}

abstract class _FallbackValues {
  _FallbackValues._();

  static const double HeadlineLargeSize = 36;
  static const double HeadlineSize = 30;
  static const double HeadlineSmallSize = 26;
  static const double BodySize = 20;
  static const double ButtonSize = 16;
  static const double TipsSize = 18;
  static const double TipsSmallSize = 16;
}
