import 'dart:ui';

import 'package:fis_common/helpers/color.dart';

/// 主题配色
class FThemeColorScheme {
  /// 主色
  final Color primary;

  /// 头部
  final Color header;

  /// 配色
  final Color secondary;

  /// 背景色
  final Color background;

  /// 导航色
  final Color nav;

  /// 线条(边框)色
  final Color line;

  /// 点缀色
  final Color additional;

  /// Dialog边框色
  final Color dialogBorder;

  /// Dialog边框色
  final Color remedicalBackground;

  /// 按钮主色
  final Color buttonPrimary;

  /// 按钮次色(取消色)
  final Color buttonSecondary;

  /// 文字色
  final Color text;

  /// 文字(选中)色
  final Color textSelected;

  /// 文字(提示文字)色
  final Color textTips;

  /// 滚动条色
  final Color scrollBar;

  /// 前景色
  final Color foreground;

  factory FThemeColorScheme({
    Color? primary,
    Color? header,
    Color? secondary,
    Color? background,
    Color? nav,
    Color? line,
    Color? additional,
    Color? dialogBorder,
    Color? remedicalBackground,
    Color? buttonPrimary,
    Color? buttonSecondary,
    Color? text,
    Color? textSelected,
    Color? textTips,
    Color? scrollBar,
    Color? foreground,
  }) {
    Color _primary = primary ??= _FallbackValues.Primary;
    Color _header = header ??= _FallbackValues.Header;
    Color _secondary = secondary ??= _FallbackValues.Secondary;
    Color _background = background ??= _FallbackValues.Background;
    Color _additional = additional ??= _FallbackValues.Additional;
    Color _nav = nav ??= _FallbackValues.Nav;
    Color _line = line ??= _FallbackValues.Line;

    Color _dialogBorder = dialogBorder ??= _FallbackValues.DialogBorder;
    Color _remedicalBackground =
        remedicalBackground ??= _FallbackValues.RemedicalBackground;
    Color _buttonPrimary = buttonPrimary ??= _FallbackValues.ButtonPrimary;
    Color _buttonSecondary =
        buttonSecondary ??= _FallbackValues.ButtonSecondary;
    Color _text = text ??= _FallbackValues.Text;
    Color _textSelected = textSelected ??= _FallbackValues.TextSelected;
    Color _textTips = textTips ??= _FallbackValues.TextTips;
    Color _scrollBar = scrollBar ??= _FallbackValues.scrollBar;
    Color _foreground = foreground ??= _FallbackValues.Foreground;

    return FThemeColorScheme.raw(
      primary: _primary,
      header: _header,
      secondary: _secondary,
      background: _background,
      additional: _additional,
      nav: _nav,
      line: _line,
      dialogBorder: _dialogBorder,
      remedicalBackground: _remedicalBackground,
      buttonPrimary: _buttonPrimary,
      buttonSecondary: _buttonSecondary,
      text: _text,
      textSelected: _textSelected,
      textTips: _textTips,
      scrollBar: _scrollBar,
      foreground: _foreground,
    );
  }

  FThemeColorScheme.raw({
    required this.primary,
    required this.header,
    required this.secondary,
    required this.background,
    required this.nav,
    required this.line,
    required this.additional,
    required this.dialogBorder,
    required this.remedicalBackground,
    required this.buttonPrimary,
    required this.buttonSecondary,
    required this.text,
    required this.textSelected,
    required this.textTips,
    required this.scrollBar,
    required this.foreground,
  });

  factory FThemeColorScheme.fromJson(Map<String, dynamic>? map) {
    if (map == null) return FThemeColorScheme();

    return FThemeColorScheme(
      primary: _convertHex2Color(map['primary']),
      header: _convertHex2Color(map['header']),
      secondary: _convertHex2Color(map['secondary']),
      background: _convertHex2Color(map['background']),
      nav: _convertHex2Color(map['nav']),
      line: _convertHex2Color(map['line']),
      additional: _convertHex2Color(map['additional']),
      dialogBorder: _convertHex2Color(map['dialogBorder']),
      remedicalBackground: _convertHex2Color(map['remedicalBackground']),
      buttonPrimary: _convertHex2Color(map['buttonPrimary']),
      buttonSecondary: _convertHex2Color(map['buttonSecondary']),
      text: _convertHex2Color(map['text']),
      textSelected: _convertHex2Color(map['textSelected']),
      textTips: _convertHex2Color(map['textTips']),
      scrollBar: _convertHex2Color(map['scrollBar']),
      foreground: _convertHex2Color(map['foreground']),
    );
  }
}

const _transparency = const Color(0x00000000);

Color? _convertHex2Color(String? hexString) {
  if (hexString == null) return null;
  if (hexString.isEmpty) return _transparency;
  return FColorHelper.hex2Color(hexString, hasTag: false);
}

abstract class _FallbackValues {
  _FallbackValues._();
  static const Primary = const Color(0xff2c77e5);
  static const Header = const Color(0xff2c77e5);
  static const Secondary = const Color(0xff7bc3ff);
  static const Nav = const Color(0xff0a2030);
  static const Additional = const Color(0xfff1413d);
  static const Line = const Color(0xffd3d3d3);
  static const Background = const Color(0xfff2f2f2);

  static const DialogBorder = const Color(0x00000000);
  static const RemedicalBackground = const Color(0xff16161a);
  static const ButtonPrimary = const Color(0xff2c77e5);
  static const ButtonSecondary = const Color(0xffd4e3f9);
  static const Text = const Color(0xff4c4948);
  static const TextSelected = const Color(0xff16161a);
  static const TextTips = const Color(0xffa5a3a3);
  static const scrollBar = const Color(0xff99a2a8);
  static const Foreground = const Color(0xffFFFFFF);
}
